unit UserBrowserDlg;

{
  Shows the familiar 'Select User' Dialog.
  This is an undocumented Dialog in netui2.dll.

  Function Call:
  SelectUsers(Title: String;                 // Dialog Caption
              hOwner: THandle;               // Handle of Owner Window
              lst: TStrings;                 // List to put selected Names in
              Filter: TBrowserIncludeFilter; // What Items to display in the Dialog
              NoLocalDomain: Boolean;        // Suppress Local Machine in Domain List
              SingleSelection: Boolean)      // allows you to select only one Name
  Result is False if User pressed Cancel, else True

  Implemented after some NG browsing by bhoc@surfeu.ch
}

interface

uses Windows, Classes;

type
  TBrowserIncludeOptions = (
    SystemAccount,
    CreatorOwner,
    Everyone,
    Interactive,
    Network,
    Users,
    UserButtons,
    Groups,
    Aliases
  );
  TBrowserIncludeFilter = Set of TBrowserIncludeOptions;

function SelectUsers(Title: String; hOwner: THandle; lst: TStrings; Filter: TBrowserIncludeFilter; NoLocalDomain : Boolean = False; SingleSelection: Boolean = False): Boolean;

implementation

const
  OPENUSERBROWSER_INCLUDE_SYSTEM       : Cardinal = $00010000;
  OPENUSERBROWSER_SINGLE_SELECTION     : Cardinal = $00001000;
  OPENUSERBROWSER_NO_LOCAL_DOMAIN      : Cardinal = $00000100;
  OPENUSERBROWSER_INCLUDE_CREATOR_OWNER: Cardinal = $00000080;
  OPENUSERBROWSER_INCLUDE_EVERYONE     : Cardinal = $00000040;
  OPENUSERBROWSER_INCLUDE_INTERACTIVE  : Cardinal = $00000020;
  OPENUSERBROWSER_INCLUDE_NETWORK      : Cardinal = $00000010;
  OPENUSERBROWSER_INCLUDE_USERS        : Cardinal = $00000008;
  OPENUSERBROWSER_INCLUDE_USER_BUTTONS : Cardinal = $00000004;
  OPENUSERBROWSER_INCLUDE_GROUPS       : Cardinal = $00000002;
  OPENUSERBROWSER_INCLUDE_ALIASES      : Cardinal = $00000001;

type
  TOpenUserBrowser = record
    cbSize: DWORD;
    Unknown1: DWORD;
    Unknown2: DWORD;
    hWndParent: THandle;
    szTitle: PWideChar;
    szDomainName: PWideChar;
    dwFlags: DWORD;
    dwHelpId: DWORD;
    szHelpFile: PWideChar;
  end;

  PTOpenUserBrowser = ^TOpenUserBrowser;

  TEnumUserBrowser = record
    SidType: SID_NAME_USE;
    Sid1: PSID;
    Sid2: PSID;
    szFullNname: PWideChar;
    szUserName: PWideChar;
    szDisplayName: PWideChar;
    szDomainName: PWideChar;
    szDescription: PWideChar;
  end;

  PTEnumUserBrowser = ^TEnumUserBrowser;

function OpenUserBrowser(lpOpenInfo: PTOpenUserBrowser): THandle; stdcall; external 'netui2.dll';

function EnumUserBrowserSelection(handle: THandle; lpBuffer: PTEnumUserBrowser; lpBufferSize: PDWORD): Boolean; stdcall; external 'netui2.dll';

function CloseUserBrowser(handle: THandle): Boolean; stdcall; external 'netui2.dll';

function SelectUsers(Title: String; hOwner: THandle; lst: TStrings; Filter: TBrowserIncludeFilter; NoLocalDomain : Boolean = False; SingleSelection: Boolean = False): Boolean;
const
  BufSize: DWORD = 2048;
var
  MyOpen: TOpenUserBrowser;
  MyEnum: TEnumUserBrowser;
  PMyOpen: PTOpenUserBrowser;
  BufPtr: Pointer;
  hDlg: THandle;
  MyUser, MyDomain: String;
  MyTitle: WideString;

begin
  Result := True;
  MyTitle := Title;
  MyOpen.cbSize := SizeOf(MyOpen);
  MyOpen.hWndParent := hOwner;
  MyOpen.szTitle := PWideChar(MyTitle);
  MyOpen.Unknown1 := 0;
  MyOpen.Unknown2 := 1;
  MyOpen.dwFlags := 0;
  if SystemAccount in Filter then MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_SYSTEM;
  if CreatorOwner in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_CREATOR_OWNER;
  if Everyone in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_EVERYONE;
  if Interactive in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_INTERACTIVE;
  if Network in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_NETWORK;
  if Users in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_USERS;
  if UserButtons in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_USER_BUTTONS;
  if Groups in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_GROUPS;
  if Aliases in Filter then  MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_INCLUDE_ALIASES;
  if SingleSelection then MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_SINGLE_SELECTION;
  if NoLocalDomain then MyOpen.dwFlags := MyOpen.dwFlags or OPENUSERBROWSER_NO_LOCAL_DOMAIN;
  MyOpen.szDomainName := '';
  PMyOpen := @MyOpen;
  GetMem(BufPtr, BufSize);
  try
    hDlg := OpenUserBrowser(pMyOpen);
    if hDlg <> 0 then
    begin
     lst.Clear;
      While EnumUserBrowserSelection(hDlg, BufPtr, @BufSize) do
      begin
        MyEnum := TEnumUserBrowser(BufPtr^);
        MyUser := MyEnum.szUserName;
        MyDomain := MyEnum.szDomainName;
        if MyDomain = '' then
          lst.Append(MyUser)
        else
          lst.Append(MyDomain + '\' + MyUser);
      end;
      CloseUserBrowser(hDlg);
    end else
    begin
      Result := False;
    end;
  finally
    FreeMem(BufPtr);
  end;
end;

end.
